#include "solver.hpp"
#include "solver_impl.hpp"

namespace solver {

std::pair<solver::GeometrySettings, std::vector<InputsMessage>>
make_geometry(solver::InputInfo ii) {
  std::vector<InputsMessage> messages;
  solver::GeometrySettings geo;

  geo.csg_filename = "";
  if (!ii.count(CSG_FILENAME)) {
    add_missing_msg(CSG_FILENAME, messages, std::vector<std::string>{""});
  } else {
    auto csg_filename = std::get<solver::StringArray>(ii[CSG_FILENAME]);
    if (csg_filename.size() != 1) {
      add_msg(CSG_FILENAME, messages, std::vector<std::string>{""});
    } else {
      geo.csg_filename = csg_filename[0];
    }
  }

  if (!ii.count(PROB_LO)) {
    add_missing_msg(PROB_LO, messages, std::vector<double>{});
  }
  if (!ii.count(PROB_HI)) {
    add_missing_msg(PROB_HI, messages, std::vector<double>{});
  }
  if (!ii.count(PERIODIC)) {
    add_missing_msg(PERIODIC, messages, std::vector<double>{0, 0, 0});
  }
  auto lows = std::get<solver::NumberArray>(ii[PROB_LO]);
  auto highs = std::get<solver::NumberArray>(ii[PROB_HI]);
  auto is_periodic = std::get<solver::NumberArray>(ii[PERIODIC]);
  if (lows.size() != 3) {
    add_msg(PROB_LO, messages, std::vector<double>{0, 0, 0});
  } else {
    std::get<0>(geo.axes).low = lows[0];
    std::get<1>(geo.axes).low = lows[1];
    std::get<2>(geo.axes).low = lows[2];
  }
  if (highs.size() != 3) {
    add_msg(PROB_HI, messages, std::vector<double>{0, 0, 0});
  } else {
    std::get<0>(geo.axes).high = highs[0];
    std::get<1>(geo.axes).high = highs[1];
    std::get<2>(geo.axes).high = highs[2];
  }
  if (is_periodic.size() != 3) {
    add_msg(PERIODIC, messages, std::vector<double>{0, 0, 0});
  } else {
    std::get<0>(geo.axes).periodic = is_periodic[0];
    std::get<1>(geo.axes).periodic = is_periodic[1];
    std::get<2>(geo.axes).periodic = is_periodic[2];
  }
  return std::make_pair(geo, messages);
}
} // namespace solver
